# import link
from . import link

link_name = "bulk loading from a local file system"
user_text = "Bulk Loading from a Local File System"
url = "https://docs.snowflake.com/en/user-guide/data-load-local-file-system.html"

link.xref_links.update({link_name: (user_text, url)})
